#####################################################################
#####################################################################
###################### Archived code for paper ######################
#####################################################################
#####################################################################

# Paper:   Synoptic processes of winter precipitation
#                 in the Upper Indus Basin
# Journal: Weather and Climate Dynamics (submitted)
# DOI:     #
# Authors: Jean-Philippe Baudouin, Michael Herzog, Cameron A. Petrie

####################################################################
# File purpose: Load libraries, data and functions
# Note:        
# Version:     08/07/2021
#####################################################################



## Main libraries
###################
  require(ggpubr)
  require(ggplot2)
  require(RColorBrewer)
  require(stringr)
  require(quantreg) # Quantile regression function rq
  require(gridExtra)
  
  
## Main datasets
##################
  ## Mask of the Indus watershed upper and lower) for different resolutions
  ##  as define in Baudouin et al 2020a and 2020b
  load("data/mask_domains.Rdata") 
  
  
  ## 3-hourly precipitation from ERA5 average over the UIB
  ##  derived from Baudouin et al 2020a and 2020b
  load("data/precip_ERA5_3h.Rdata") 

  
  # ERA5 relief from Data.R
  if(exists("data/data_relief.Rdata")) load("data/data_relief.Rdata")
  
  g <- 9.80665 # To convert geopotential into geopotential heights
  
################################################################################
################################################################################

####################################  
########## Main functions ##########
####################################
  
  ## Plot ascetics
  ##################
  
  # Function plotting the contour of the UIB on a ggplot
  mask_up <- function(colour = "black", size=1.5) {
    geom_contour(data=mask, aes(x=x, y=y, z=up), breaks=.5, colour = colour, size=size)
  }
  
  
  ## Blank ggplot for titles in multipanel plots
  blank_ggplot <- function(text, size = 10, rot = F,...) {
    p <- ggplot() + 
      theme(panel.background = element_blank(),
            axis.ticks = element_blank(),
            plot.margin  = unit(c(0,0,0,0), units = "pt")) +
      labs(x = NULL, y = NULL) + 
      scale_x_continuous(labels = NULL) + 
      scale_y_continuous(labels = NULL)
    
    if(rot) p <- p + annotate("text", x= 0, y=0, label = text, size = size, angle = 90, ...)
    else    p <- p + annotate("text", x= 0, y=0, label = text, size = size, ...)
    
    return(p)
    
  }
  
  
  ## Compute seasonal anomaly
  # This is done by removing the "h" first harmonics of the seasonal cycle using a fft.
  # x is the time series and ny the number of years
  # The number of years in x needs to be an integer
  unseas <- 
    function(x, ny, h=2) {
      X=fft(x)
      sel <- sapply(1:h, function(i) c(i*ny,length(x)-i*ny ))
      X[c(0, sel) +1] <- 0
      y <- Re(fft(X, inv=T)/length(x))
      gc()
      return(y)
    }
  
  
  ## Remove the legend from a ggplot
  noleg <- theme(legend.position="none")
  
  
  ## Theme for map
  theme_map <- theme(
    panel.background = element_rect(fill ="grey80"),
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    
    plot.title   = element_text(size = 24, face = "bold",
                                margin = margin(0,0,20,0), hjust = 0.5),
    plot.margin  = margin(30,10,7.5,0),
    
    legend.text  = element_text(size = 15),
    legend.title = element_text(size = 16),
    legend.key.size = unit(1, "cm"),
    legend.key   = element_rect(fill = "white", colour = "white", size = 1),
    legend.position="bottom",
    
    axis.text    = element_text(size = 14),
    axis.title   = element_text(size = 20),
    axis.title.x = element_text(margin = margin(20,0,0,0)),
    axis.title.y = element_text(margin = margin(0,20,0,0)))
  
  
  ## Theme for plot
  theme_plot <- theme(
    panel.background = element_rect(fill = "transparent", colour= "black"),
    panel.grid.major = element_line(colour = "grey50"),
    panel.grid.minor = element_line(colour = "grey70"),
    
    plot.title   = element_text(size = 24, face = "bold",
                                margin = margin(0,0,20,0), hjust = 0.5),
    plot.margin  = unit(c(1,1,1,1), units = "cm"),
    
    legend.text  = element_text(size = 16),
    legend.title = element_text(size = 16),
    legend.key.size = unit(1, "cm"),
    legend.key   = element_rect(fill = "white", colour = "white", size = 1),
    
    axis.text    = element_text(size = 14),
    axis.title   = element_text(size = 20),
    axis.title.x = element_text(margin = margin(20,0,0,0)),
    axis.title.y = element_text(margin = margin(0,20,0,0))
  )
  
  
  
  
  ## Ploting functions
  ######################
  
  ## Plot composite of geopotential 300hPa (z) and geopotential thickness 500-300hPa (dz)
  ## The timesteps to average on is defined by sel
  ## seqz, seqdz corresponds to the isolines of z and dz respectively and also
  ##    indicate the max/min values of the fields (i.e all values above/below the
  ##    threshold will have the same color)
  plot_WD <- function(z, dz, sel, seqz, seqdz) {
    data_z <- data.frame(value = apply(z$unseas[,sel], 1, mean)/g, x = z$map[,1], y = z$map[,2],
                         pval = apply(z$unseas[,sel], 1, function(x) t.test(x)$p.value))
    data_z$value2 <- data_z$value
    data_z$value2[which(data_z$pval>0.05)] <- 0
    
    data_mask <- data.frame(x = z$map[-z$sel,1], y = z$map[-z$sel,2])
    
    data_dz <- data.frame(value = apply(dz$unseas[,sel], 1, mean)/g, x = dz$map[,1], y = dz$map[,2])
    
    print(data_z[which.min(data_z$value),])
    
    p <- ggplot() +
      geom_raster(data=data_z,   aes(x, y, fill=pmin(pmax(value2, min(seqz)), max(seqz))), interpolate = T) +
      geom_contour(data=data_z, aes(x, y, z=value, colour=..level..), breaks=seqz, size=.4, linetype = "22", colour = "grey30") +
      geom_contour(data=data_z, aes(x, y, z=value, colour=..level..), breaks=0, size=.4, colour = "grey30") +
      geom_raster(data=data_relief, aes(x=x, y=y, alpha=z), show.legend=F, interpolate=T) +
      geom_raster(data=data_mask, aes(x=x, y=y), show.legend=F, interpolate = T, fill="grey40") +
      geom_contour(data=data_dz, aes(x=x, y=y, z=value, colour=..level..), breaks=seqdz, size=1.5) +
      geom_contour(data = data_relief, aes(x = x, y = y, z = lsm), breaks = 0.5, colour="black", size=.7) +
      theme_map + coord_quickmap() + mask_up(size=1) + 
      labs(x=NULL, y=NULL) + theme(legend.position="bottom") +
      theme(plot.margin  = margin(15,10,15,0)) + 
      scale_alpha_continuous(range=c(0,.8)) +
      scale_x_continuous(expand=c(0,-.5), limits=c(49,91), breaks=seq(45,90,5)) + 
      scale_y_continuous(expand=c(0,-.5), limits=c(19,51), breaks=seq(10,50,5)) +
      scale_colour_gradientn(
        colours =  brewer.pal(name="RdYlBu", n=11), values = seq(1,0,len=11),
        breaks = seqdz, limits=c(-max(abs(seqdz)),max(abs(seqdz))),
        name  = expression(bold(atop("Geopotential thickness", "anomaly 500-300hPa (gpm)"))),
        guide = guide_legend(title.position = "top", title.hjust=.5,  order = 2,
                             label.position = "bottom", nrow=1, keyheight = 1)) +
      scale_fill_gradient2(
        low=brewer.pal(name="RdBu", n=7)[7], mid="white", high=brewer.pal(name="RdBu", n=7)[1],
        breaks = seqz, limits=range(seqz),
        name  = expression(bold(atop("Geopotential height", " anomaly at 300hPa (gpm)"))),
        guide = guide_colourbar(title.position = "top", title.hjust = 0.5, title.vjust = 0.5,
                                label.theme = element_text(angle = 30, size = 16), order = 1,
                                label.hjust = 1, label.vjust = 1.2, 
                                ticks.colour = "grey30", ticks.linewidth = 1,
                                barwidth = 26, barheight = 1))
  }
  
  
  ## Composite plot of temperature and geopotential anomaly over a longitudinal cross-section
  # sel is the selection on which the fields are averaged
  plot_vert <- function(sel) {
    
    lon <- z32$map$x # x axis
    
    val <- apply(z32$unseas[sel,,], 2:3, mean)/g
    alt <- apply(z32$data[,sel,], c(1,3), mean)
    # The line below interpolates the values so that the steps on the y-axis are
    #     constant (in meter geopotential, not in pressure level)
    val_alt <- sapply(1:51, function(i) approx(alt[i,]/9.81, y=val[i,], xout=alt_out, rule=2)$y, simplify = "array")
    data_z <- data.frame(value = c(val_alt), x = rep(lon, each=length(alt_out)), y = rep(as.integer(alt_out), length(lon)))
    
    # We compute the minimum of geopotential anomaly at each anomaly. It is then smoothed
    #   to produce the thick, black vertical line on the plot
    val_alt[1:9, 47:51] <- 0
    data_min <- data.frame(lon = lon[apply(val_alt, 1, which.min)], alt = alt_out)
    data_min3 <- data.frame(ksmooth(data_min$alt , data_min$lon, n.points=1000, band=2000, k="normal"))
    
    val <- apply(t32$unseas[sel,,], 2:3, mean)
    alt <- apply(z32$data[,sel,], c(1,3), mean)
    val_alt <- sapply(1:51, function(i) approx(alt[i,]/9.81, y=val[i,], xout=alt_out, rule=2)$y, simplify = "array")
    data_t <- data.frame(value = c(val_alt), x = rep(lon, each=length(alt_out)), y = rep(as.integer(alt_out), length(lon)))
    
    alt300 <- apply(z32$data[,sel,8], 1, mean)/9.81
    alt700 <- apply(z32$data[,sel,16], 1, mean)/9.81
    
    seqz <- seq(-105,30,15)
    
    
    p <- ggplot() +
      geom_raster(data=data_z,   aes(x, y, fill=pmin(pmax(value, min(seqz)), max(seqz))), interpolate = T) +
      geom_contour(data=data_z, aes(x, y, z=value, colour=..level..), breaks=seqz, size=.4, linetype = "22", colour = "grey30") +
      geom_contour(data=data_z, aes(x, y, z=value, colour=..level..), breaks=0, size=.4, colour = "grey30") +
      geom_hline(yintercept = alt700[51], linetype="12", size=.8) +
      geom_hline(yintercept = alt300[51], linetype="12", size=.8) +
      geom_path(data=data_min3, aes(x=y, y=x), size=1) +
      theme_map + theme(legend.position="bottom") +
      theme(plot.margin  = margin(15,10,15,0)) + 
      geom_contour(data=data_t, aes(x=x, y=y, z=value, colour=..level..), breaks=seq(-5,2,1), size=1.5) +
      labs(x=NULL, y=NULL, title=NULL)  +
      scale_x_continuous(expand=c(0,-2), limits=c(53,82), breaks=seq(56,90,4)) + 
      scale_y_continuous(expand=c(0,-500), limits=c(-500,15500), breaks=seq(0,15000,2000), sec.axis = sec_axis(~., name = NULL, breaks=apply(alt/9.81, 2, mean)[sel_alt], labels=z32$alt[sel_alt])) +
      geom_polygon(data=data.frame(y=c(0, surf32,0), x=c(55, seq(55,80,.25), 80)), aes(x=x, y=y), fill="grey80", colour="grey20") + 
      scale_colour_gradientn(
        colours = brewer.pal(name="RdYlBu", n=11), values = seq(1,0,len=11), 
        breaks=seq(-5,2,1), limits=c(-5,5),
        name  = expression(bold("Temperature anomaly (K)"^phantom(2))),
        guide = guide_legend(title.position = "top", title.hjust=.5, order = 2,
                             label.position = "bottom", nrow=1,
                             keyheight = 1)) +
      scale_fill_gradient2(
        low=brewer.pal(name="RdBu", n=7)[7], mid="white", high=brewer.pal(name="RdBu", n=7)[1],
        name = expression(bold("Geopotential height anomaly (gpm)")),
        limits = range(seqz), breaks = seqz,
        guide = guide_colourbar(title.position = "top", title.hjust = 0.5, title.vjust = 0.5,
                                label.theme = element_text(angle = 30, size = 16), order = 1,
                                label.hjust = 1, label.vjust = 1.2, 
                                ticks.colour = "grey30", ticks.linewidth = 1,
                                barwidth = 20, barheight = 1))
    
  }
  
  
  
  ## Same as plot_vert but for meridional wind and moisture anomalies
  plot_vert2 <- function(sel) {
    lon <- z32$map$x
    
    val <- apply(v32$unseas[sel,,], 2:3, mean)
    alt <- apply(z32$data[,sel,], c(1,3), mean)
    val_alt <- sapply(1:51, function(i) approx(alt[i,]/9.81, y=val[i,], xout=alt_out, rule=2)$y, simplify = "array")
    data_v <- data.frame(value = c(val_alt), x = rep(lon, each=length(alt_out)), y = rep(as.integer(alt_out), length(lon)))
    
    val <- apply(q32$unseas[sel,,], 2:3, mean)
    alt <- apply(z32$data[,sel,], c(1,3), mean)
    val_alt <- sapply(1:51, function(i) approx(alt[i,]/9.81, y=val[i,], xout=alt_out, rule=2)$y, simplify = "array")
    data_q <- data.frame(value = c(val_alt)*1000, x = rep(lon, each=length(alt_out)), y = rep(as.integer(alt_out), length(lon)))
    
    alt300 <- apply(z32$data[,sel,8], 1, mean)/9.81
    alt700 <- apply(z32$data[,sel,16], 1, mean)/9.81
    
    seqv <- seq(-10, 14, 2)
    seqQ <- seq(-0.0008, 0.0020, 0.0004)*1000
    
    p <- ggplot() +
      geom_raster(data=data_v,   aes(x, y, fill=pmin(pmax(value, min(seqv)), max(seqv))), interpolate = T) +
      geom_contour(data=data_v, aes(x, y, z=value, colour=..level..), breaks=seqv, size=.4, linetype = "22", colour = "grey30") +
      geom_contour(data=data_v, aes(x, y, z=value, colour=..level..), breaks=0, size=.4, colour = "grey30") +
      geom_hline(yintercept = alt700[51], linetype="11", size=1) +
      geom_hline(yintercept = alt300[51], linetype="11", size=1) +
      theme_map + theme(legend.position="bottom") +
      theme(plot.margin  = margin(15,10,15,0)) + 
      geom_contour(data=data_q, aes(x=x, y=y, z=value, colour=..level..), breaks=seqQ, size=1.5) +
      labs(x=NULL, y=NULL, title=NULL)  +
      scale_x_continuous(expand=c(0,-2), limits=c(53,82), breaks=seq(56,90,4)) + 
      scale_y_continuous(expand=c(0,-500), limits=c(-500,15500), breaks=seq(0,15000,2000), sec.axis = sec_axis(~., name = NULL, breaks=apply(alt/9.81, 2, mean)[sel_alt], labels=z32$alt[sel_alt])) +
      geom_polygon(data=data.frame(y=c(0, surf32,0), x=c(55, seq(55,80,.25), 80)), aes(x=x, y=y), fill="grey80", colour="grey20") + 
      scale_colour_gradientn(
        colours = brewer.pal(name="BrBG", n=11), values = seq(0,1,len=11), 
        breaks=seqQ, limits=range(-max(abs(seqQ)), max(abs(seqQ))),
        name  = expression(bold("Specific moisture anomaly (kg."*kg^"-1"*".10"^"-3"*")")),
        guide = guide_legend(title.position = "top", title.hjust=.5, order = 2,
                             label.position = "bottom", nrow=1,
                             keyheight = 1)) +
      scale_fill_gradient2(
        low=brewer.pal(name="PuOr", n=5)[5], mid="white", high=brewer.pal(name="PuOr", n=5)[1],
        name = expression(bold("Meridional wind anomaly (m."*s^"-1"*")")),
        limits = range(seqv), breaks = seqv,
        guide = guide_colourbar(title.position = "top", title.hjust = 0.5, title.vjust = 0.5,
                                label.theme = element_text(angle = 30, size = 16), order = 1,
                                label.hjust = 1, label.vjust = 1.2, 
                                ticks.colour = "grey30", ticks.linewidth = 1,
                                barwidth =20, barheight = 1))
    
  }
  
  
  
  ## Composite plot of wind speed and wind speed anomaly at 250hPa
  # sel is the selection on which the fields are averaged
  plot_jet2 <-  function(sel, title=NULL) {
    data <- data.frame(value = apply(wind250$data[,sel], 1, mean),
                       value2 = apply(wind250$unseas[,sel], 1, mean),
                       pval = apply(wind250$unseas[,sel], 1, function(x) t.test(x)$p.value),
                       x = wind250$map[,1], y = wind250$map[,2])
    
    data$value3 <- data$value2
    data$value3[which(data$pval>0.05)] <- 0
    
    breaks <- seq(-10, 10, 2)
    p <- ggplot(data, aes(x=x, y=y)) + 
      geom_raster(aes(fill=pmax(pmin(value3, max(breaks)), min(breaks))), interpolate=T) +
      geom_raster(data=data_relief, aes(x=x, y=y, alpha=z), show.legend=F, interpolate=T) +
      geom_contour(aes(z=value2), linetype="21", size=.5, breaks=breaks, color="grey30") +
      geom_contour(aes(z=value2), size=.5, breaks=0, color="grey30") +
      geom_contour(data = data_relief, aes(x = x, y = y, z = lsm), breaks = 0.5, colour="black", size=1) +
      geom_contour(aes(z=value, colour=..level..), size = 1, breaks=seq(0,55,5)) +
      mask_up(size=1) + theme_map + coord_quickmap() +
      labs(x=NULL, y=NULL, title=title) +   
      scale_x_continuous(expand=c(0,-5), limits=c(-25, 155), breaks = seq(-20,150,20)) +
      scale_y_continuous(expand=c(0,-5), limits=c(5, 75)) +
      scale_alpha_continuous(range=c(0,.8)) +
      scale_fill_gradientn(
        colours = rev(brewer.pal("RdBu", n=11)), values = seq(0,1, .1),
        limits = range(breaks), breaks = breaks,
        name  = expression(bold("Wind speed anomaly at 250hPa (m."*s^"-1"*")")),
        guide = guide_colorbar(barwidth = 20, barheight = 1,
                               title.position = "top", title.hjust=.5,
                               ticks.colour = "grey50",
                               ticks.linewidth = 2,
                               label.hjust = .5, label.vjust =1.3,
                               order=2)) +
      scale_colour_gradientn(colours  = c("white", "#f7fcb9", "#78c679", "#004529"),
                             na.value = "white", 
                             limits   = c(0,1)*55, breaks   = seq(10,55,5),
                             name     = expression(bold("Wind speed at 250hPa (m."*s^"-1"*")")),
                             guide = guide_legend(title.position = "top", title.hjust=.5,
                                                  label.position = "bottom", nrow=1,
                                                  ticks.colour = "grey30", ticks.linewidth = 1,
                                                  keyheight = 1, order=1))
  }
  
  
  
  ## Plot the regression coefficients provided by data_reg
  plot_reg <- function(data_reg, max, label = "A)", x=54) {
    plot_field(data_reg, range(data_reg$x), range(data_reg$y), seq(-1,1,2)*max, "anom", legend=NULL, bg_map=F, North = F, river = F, interpol = F, ocean = F, indus = F) +
      coord_quickmap() + mask_up(size=1) +
      geom_contour(data = data_relief, aes(x = x, y = y, z = lsm), breaks = 0.5, colour="black", size=.7) +
      geom_contour(data = data_relief, aes(x = x, y = y, z = z), breaks = c(10000, 30000, 50000), colour="grey30", size=.3) +
      scale_x_continuous(expand=c(0,-.5), limits=c(49,91), breaks=seq(45,90,5)) + 
      scale_y_continuous(expand=c(0,-.5), limits=c(19,51), breaks=seq(10,50,5)) +
      labs(x=NULL, y=NULL, title=NULL) +
      theme(legend.position="bottom", plot.margin  = margin(30,10,7.5,0)) + theme1 + theme + 
      scale_fill_gradientn(
        colours=brewer.pal(name="RdBu", n=11), values = seq(1,0,-.1),
        name = NULL, limits = c(-max,max), breaks = c(-max/3*2, 0, max/3*2),
        labels = c(as.expression(bquote(beta~"< 0")),"0   ", as.expression(bquote(beta~"> 0"))),     
        guide = guide_colourbar(barwidth = 20, barheight = 1, ticks=F)) +
      annotate("label", x=x, y=47.5, label=label, size=10, label.padding = unit(c(.6), "lines"), alpha=.5)    
  }
  
  
  ## Scatterplot
  plot_pts <- function(data_pts, R2, label = "B)") {
    p_pts <- ggplot(data_pt) +
      geom_point(aes(x=v, y=z), size=.5) +
      geom_hline(aes(yintercept=0), color="grey20", size=1) +
      geom_vline(aes(xintercept=0), color="grey20", size=1) +
      theme1 + coord_equal() + theme(axis.title = element_text(size = 22)) +
      geom_abline(slope=1, col="darkgreen", size=1) +
      scale_x_continuous(expand = c(0,0), limits=c(-5,10), breaks=seq(-5,10,5)) +
      scale_y_continuous(expand = c(0,0), limits=c(-5,10), breaks=seq(-5,10,5)) +
      labs(x=expression("W700 (m."*s^"-1"*")"), y="Prediction with Z300") +
      annotate("label", x=6, y=-2.4, label=paste("R^2 == ", R2), parse=T, size=7, label.padding = unit(c(.4), "lines"), label.size = 0) +
      annotate("label", x=-3.15, y=8.5, label=label, size=10, label.padding = unit(c(.6), "lines"))
  }
  
  
  
  
  ## Composite plot of precipitable water and moisture transport
  # sel is the selection on which the fields are averaged
  plot_PW <- function(sel) {
    
    data <- data.frame(value = apply(PW$unseas[, sel], 1, mean), x = PW$map[,1], y = PW$map[,2],
                       pval = apply(PW$unseas[,sel], 1, function(x) t.test(x)$p.value))
    data$value2 <- data$value
    data$value2[which(data$pval>0.05)] <- 0 # Non-significant values are removed
    
    dataW <- data.frame(u = apply(um$data[, sel], 1, mean),
                        v = apply(vm$data[, sel], 1, mean),
                        x = vm$map[,1], y = vm$map[,2])
    
    breaks <- seq(-4, 10, 1)
    
    ggplot(data, aes(x=x, y=y)) + 
      geom_raster(aes(fill=pmax(pmin(value2, max(breaks)), min(breaks))), interpolate=T) +
      geom_raster(data=data_relief, aes(x=x, y=y, alpha=z), show.legend=F, interpolate=T) +
      geom_contour(aes(z=value), linetype="21", size=.5, breaks=breaks, color="grey30") +
      geom_contour(aes(z=value), size=.5, breaks=0, color="grey30") +
      geom_contour(data = data_relief, aes(x = x, y = y, z = lsm), breaks = 0.5, colour="black", size=1) +
      geom_segment(data=dataW,aes(x = x, y = y, xend=x+u*facW1, yend=y+v*facW1), arrow = arrow(length = unit(0.15,"cm"))) +
      mask_up(size=1) +
      theme_map + noleg + coord_quickmap() +
      theme(plot.margin = margin(.5,.5,.5,.5,"cm")) +
      labs(x=NULL, y=NULL, title=NULL) +   
      scale_x_continuous(expand=c(0,-4.5), limits=c(min(data$x)-5, max(data$x)+5), breaks=seq(30,100,10)) +
      scale_y_continuous(expand=c(0,-4.5), limits=c(min(data$y)-5, max(data$y)+5)) +
      scale_alpha_continuous(range=c(0,.8)) +
      scale_fill_gradientn(colors = c("brown3", "white", "dodgerblue3", "darkblue"), values = c(0,4,8,14)/14,
                           limits = range(breaks), breaks = breaks,
                           name  = expression(bold("Precipitable water anomaly (mm)")),
                           guide = guide_colorbar(barwidth = 25, barheight = 1,
                                                  title.position = "top", title.hjust=.5,
                                                  ticks.colour = "grey30",
                                                  ticks.linewidth = 2,
                                                  label.hjust = .5, label.vjust = 1.3))
  } 
  
  
  
  ## Composite plot of evaporation and 10m sea surface wind
  # sel is the selection on which the fields are averaged
  plot_ET <- function(sel) {
    
    data <- data.frame(value = -apply(ET$unseas[, sel], 1, mean)*1000, x = ET$map[,1], y = ET$map[,2],
                       pval = apply(ET$unseas[,sel], 1, function(x) t.test(x)$p.value))
    for (i in 1:7) data$value <- data$value - apply(ET$unseas[, sel-i], 1, mean)*1000
    
    dataW <- data.frame(u = apply(u10$data[, sel], 1, mean),
                        v = apply(v10$data[, sel], 1, mean),
                        x = v10$map[,1], y = v10$map[,2])
    sel_sea <- which(data_relief$lsm[which(data_relief$x %in% unique(dataW$x) & data_relief$y %in% unique(dataW$y))] < .5)
    
    breaks <- seq(-3, 3, 1)
    
    ggplot(data, aes(x=x, y=y)) + 
      geom_raster(aes(fill=pmax(pmin(value, max(breaks)), min(breaks))), interpolate=T) +
      geom_raster(data=data_relief, aes(x=x, y=y, alpha=z), show.legend=F, interpolate=T) +
      geom_contour(aes(z=value), linetype="21", size=.5, breaks=breaks, color="grey30") +
      geom_contour(aes(z=value), size=.5, breaks=0, color="grey30") +
      geom_contour(data = data_relief, aes(x = x, y = y, z = lsm), breaks = 0.5, colour="black", size=1) +
      geom_segment(data=dataW[sel_sea,],aes(x = x, y = y, xend=x+u*facW1, yend=y+v*facW1), arrow = arrow(length = unit(0.15,"cm"))) +
      mask_up(size=1) +
      theme_map + noleg + coord_quickmap() +
      theme(plot.margin = margin(.5,.5,.5,.5,"cm")) +
      labs(x=NULL, y=NULL, title=NULL) +   
      scale_x_continuous(expand=c(0,-4.5), limits=c(min(data$x)-5, max(data$x)+5), breaks=seq(30,100,10)) +
      scale_y_continuous(expand=c(0,-4.5), limits=c(min(data$y)-5, max(data$y)+5)) +
      scale_alpha_continuous(range=c(0,.8)) +
      scale_fill_gradient2(low = "brown3", mid = "white", high = "dodgerblue3", na.value = "grey",
                           limits = range(breaks), breaks = breaks,
                           name  = expression(bold(atop("Evaporation anomaly","in the last 24 hours (mm)"))),
                           guide = guide_colorbar(barwidth = 25, barheight = 1,
                                                  title.position = "top", title.hjust=.5,
                                                  ticks.colour = "grey50",
                                                  ticks.linewidth = 2,
                                                  label.hjust = .5, label.vjust = 1.3))
  }
  
  
  
  
  
  
  
  