#####################################################################
#####################################################################
###################### Archived code for paper ######################
#####################################################################
#####################################################################

# Paper:   Synoptic processes of winter precipitation
#                 in the Upper Indus Basin
# Journal: Weather and Climate Dynamics (submitted)
# DOI:     #
# Authors: Jean-Philippe Baudouin, Michael Herzog, Cameron A. Petrie

####################################################################
# File purpose: Created the .Rdata files from netCDF
# Note:        
# Version:     09/07/2021
####################################################################



## Description of the data files needed:
##########################################
## ERA5 1979-2018, one file per year


## 3D wind and moisture:
# variables        = u, v, q      
# altitudes        = 850, 700
# time resolution  = 3h
# space resolution = .5°
# space domain     = 67/85°E, 28/36°N
# name convention  = ERA5_uvq_YEAR.nc


## 3D geopotential:
# variables        = z
# altitudes        = 850, 700, 500, 300
# time resolution  = 3h
# space resolution = 1°
# space domain     = 50/90°E, 20/50°N
# name convention  = ERA5_z_YEAR.nc


## 2D altitude wind:
# variables        = u, v
# altitudes        = 250
# time resolution  = 3h
# space resolution = 1°
# space domain     = -20/150°E, 10/70°N
# name convention  = ERA5_uv250_YEAR.nc


## 2D surface: (precipitable water, vertically integrated moisture transport East and North, evaporation, 10m surface wind East and North)
# variables        = tcwv (pw) p71.162(um), p72.162(vm), e, u10, v10
# altitudes        = surface
# time resolution  = 3h
# space resolution = 1°
# space domain     = 10/100°E, 10/50°N
# name convention  = ERA5_water_YEAR.nc


## Cross-section variables, ! one file per year and per altitude
# variables        = z t v q
# altitudes        = 100 to 250 per 25 hPa, 250 to 800 per 50hPa, 800 to 100 per 25hPa
# (it corresponds to all altitudes available below 100hPa)
# time resolution  = 3h
# space resolution = .5°
# space domain     = 55/80°E, 20/40°N
# (in fact only 32°N is used, but the function load_vert needs a 4 dimension matrix, 
#  so at least 2 latitudes need to be download)
# name convention  = ERA5_vert_YEAR_ALTITUDE.nc


## Altitude (geopotential) and land-sea mask
# variables        = z, lsm
# altitudes        = surface
# time resolution  = fixed
# space resolution = .25°
# space domain     = global
# name convention  = altitude.nc



## libraries, sources
#######################

  data_path <- "/" # path where to find the ERA5 netcdf files
  
  source("01_functions.R")
  require(akima)
  library(zoo)
  require(abind)
  require(ncdf4)


###############################
########## Functions ##########
###############################
  
  
  ## Retrieves altitude of the ERA5 surface in geopotential
  ###########################################################
  
  ag_era5 <- function(lat, lon) {
    
    require(ncdf4)
    
    # lat, lon: list of latitude and longitudes to retrieve
    
    file <- nc_open(paste0(data_path, "altitude.nc"))
    lon_alt <- ncvar_get(file, "longitude")
    lat_alt <- ncvar_get(file, "latitude")
    alt_geop <- ncvar_get(file, "z")
    nc_close(file)
    
    sel_lon <- which(lon_alt %in% lon)
    sel_lat <- which(lat_alt %in% lat)
    alt_geop <- alt_geop[sel_lon, sel_lat]
    
    return(alt_geop)
  }
  
  
  ## Retrieves 3D fields 
  ########################
  
  load_data <- function(var, years, lat, lon, alt) {
    
    # var: name of the variable: u, v, z, q
    # years: 1979-2018
    # lat, lon: list of latitude and longitudes to retrieve
    # alt: pressure level in hPa
    
    data <- c()
    time <- c()
    
    for (y in years) {
      print(y)

      if (var == "z") {
        file <- nc_open(paste0(data_path, "ERA5_z_",y,".nc"))
      } else if (var == "u" | var == "v" | var == "q") {
        file <- nc_open(paste0(data_path, "ERA5_uvq_",y,".nc"))
      } else stop("wrong variable")
      
      lonf <- ncvar_get(file, "longitude")
      latf <- ncvar_get(file, "latitude")
      altf <- ncvar_get(file, "level")
      sel_lon <- which(lonf >= min(lon) & lonf <= max(lon))
      sel_lat <- which(latf >= min(lat) & latf <= max(lat))
      sel_alt <- which(altf %in% alt)
      
      time_tmp <- as.POSIXct(ncvar_get(file, "time")*3600, origin = "1900-01-01 00:00:00", tz = "GMT")
      datay <- ncvar_get(file, var, start = c(sel_lon[1],sel_lat[1],sel_alt,1), count = c(length(sel_lon), length(sel_lat),1,-1))
      
      sel_lon2 <- which(lonf[sel_lon] %in% lon)
      sel_lat2 <- which(latf[sel_lat] %in% lat)
      
      map <- data.frame(x=rep(lonf[sel_lon[sel_lon2]], length(lat)), y=rep(latf[sel_lat[sel_lat2]], each=length(lon)))
      
      data <- cbind(data, matrix(datay[sel_lon2, sel_lat2,],length(lon)*length(lat),length(time_tmp)))
      
      time <- c(time, time_tmp)
      
      nc_close(file)
      
      gc()
      
    }
    return(list(data=data, time=as.POSIXct(time, origin = "1970-01-01 00:00:00"), map=map))
  }
  
  
  
  ## Retrieves 2D fields 
  ########################
  
  load_data2D <- function(var, years, lat, lon, alt) {
    
    # var: name of the variable: u, v, z, q
    # years: 1979-2018
    # lat, lon: list of latitude and longitudes to retrieve
    # alt: pressure level in hPa or "surface"
    
    data <- c()
    time <- c()
    
    for (y in years) {
      print(y)
      if ((var == "u" | var == "v") & alt == "250") {
        file <- nc_open(paste0(data_path, "ERA5_uv250_",y,".nc"))
      } else if (var == "tcwv" | var == "e" | var == "u10" | var == "v10" | var == "p71.162" | var == "p72.162") {
        file <- nc_open(paste0(data_path, "ERA5_water_",y,".nc"))
      } else stop("wrong variable")
      
      lonf <- ncvar_get(file, "longitude")
      latf <- ncvar_get(file, "latitude")
      
      
      sel_lon <- which(lonf >= min(lon) & lonf <= max(lon))
      sel_lat <- which(latf >= min(lat) & latf <= max(lat))
      
      time_tmp <- as.POSIXct(ncvar_get(file, "time")*3600, origin = "1900-01-01 00:00:00", tz = "GMT")
      
      datay <- ncvar_get(file, var, start = c(sel_lon[1],sel_lat[1],1), count = c(length(sel_lon), length(sel_lat),-1))
      
      datay <- array(datay, dim = c(length(sel_lon), length(sel_lat), length(time_tmp)))
      
      if (min(lon) < 0) {
        
        sel_lonn <- which(lonf >= min(lon)+360)
        datayn <- ncvar_get(file, var, start = c(sel_lonn[1],sel_lat[1],1), count = c(length(sel_lonn), length(sel_lat),-1))
        
        sel_lon2 <- which(c(lonf[sel_lonn] - 360, lonf[sel_lon]) %in% lon)
        sel_lat2 <- which(latf[sel_lat] %in% lat)
        
        map <- data.frame(x=rep(lon, length(lat)), y=rep(latf[sel_lat[sel_lat2]], each=length(lon)))
        data <- cbind(data, matrix(abind(datayn, datay, along = 1)[sel_lon2, sel_lat2,],
                                   length(lon)*length(lat),length(time_tmp)))
        
        
      } else {
        
        sel_lon2 <- which(lonf[sel_lon] %in% lon)
        sel_lat2 <- which(latf[sel_lat] %in% lat)
        
        map <- data.frame(x=rep(lonf[sel_lon[sel_lon2]], length(lat)), y=rep(latf[sel_lat[sel_lat2]], each=length(lon)))
        
        data <- cbind(data, matrix(datay[sel_lon2, sel_lat2,],length(lon)*length(lat),length(time_tmp)))
        
      }
      
      time <- c(time, time_tmp)
      
      nc_close(file)
      gc()
      
    }
    return(list(data=data, time=as.POSIXct(time, origin = "1970-01-01 00:00:00"), map=map))
  }
  
  
  
  ## Retrieves Vertical fields 
  ##############################
  
  
  load_vert <- function(var, years, lat, lon) {
    
    # var: name of the variable: z t v q
    # years: 1979-2018
    # lat, lon: min/max couple of latitude and longitudes to retrieve
    # alt: pressure level in hPa or "surface"
    
    data <- c()
    time <- c()
    
    # All the altitude available
    alt <- c('100','125','150','175','200','225','250','300','350','400','450','500', '550','600','650','700','750','775','800','825','850','875','900','925', '950','975','1000')
    
    for(a in alt) {
      data_a <- c()
      for (y in years) {
        print(paste(y, a))
        file <- nc_open(paste0(data_path, "ERA5_vert_",y,"_",a,".nc"))
        
        
        lonf <- ncvar_get(file, "longitude")
        latf <- ncvar_get(file, "latitude")
        sel_lon <- which(lonf >= min(lon) & lonf <= max(lon))
        sel_lat <- which(latf >= min(lat) & latf <= max(lat))
        
        time_tmp <- as.POSIXct(ncvar_get(file, "time")*3600, origin = "1900-01-01 00:00:00", tz = "GMT")
        datay <- ncvar_get(file, var, start = c(sel_lon[1],sel_lat[1],1), count = c(length(sel_lon), length(sel_lat),-1))
        
        
        map <- data.frame(x=rep(lonf[sel_lon], length(sel_lat)), y=rep(latf[sel_lat], each=length(sel_lon)))
        
        data_a <- cbind(data_a, matrix(datay,length(sel_lon)*length(sel_lat),length(time_tmp)))
        
        time <- c(time, time_tmp)
        
        nc_close(file)
        gc()
        
      }
      
      data <- abind(data, data_a, along=3)
    }
    
    return(list(data=data, time=as.POSIXct(time, origin = "1970-01-01 00:00:00"), map=map, alt=alt))
  }
  
  
  
###################################################
########## Relief and land sea mask data ##########
###################################################

  file <- nc_open(paste0(data_path, "altitude.nc"))
  lon <- ncvar_get(file, "longitude")
  lat <- ncvar_get(file, "latitude")
  alt_geop <- ncvar_get(file, "z")
  alt_geop2 <- c(rbind(alt_geop, alt_geop))
  lsm <- ncvar_get(file, "lsm")
  lsm2 <- c(rbind(lsm, lsm))
  
  nc_close(file)
  
  map_era5 <- cbind(rep(c(lon-360, lon), length(lat)), rep(lat, each=length(lon)*2))
  data_relief <- data.frame(z = alt_geop2, lsm = lsm2, x = map_era5[,1], y = map_era5[,2])
  

  # some lakes are removed
  data_relief$lsm[which(data_relief$lsm<.5 & data_relief$x > 40 & data_relief$x < 47 & data_relief$y > 30 & data_relief$y < 40)] <- .5
  data_relief$lsm[which(data_relief$lsm<.5 & data_relief$x > 30 & data_relief$x < 103 & data_relief$y > 48 & data_relief$y < 59)] <- .5
  data_relief$lsm[which(data_relief$lsm<.5 & data_relief$x > 85 & data_relief$x < 90 & data_relief$y > 65 & data_relief$y < 70)] <- .5
  data_relief$lsm[which(data_relief$lsm<.5 & data_relief$x > 70 & data_relief$x < 105 & data_relief$y > 30 & data_relief$y < 50)] <- .5
  data_relief$lsm[which(data_relief$lsm<.5 & data_relief$x > 30 & data_relief$x < 35 & data_relief$y > 37 & data_relief$y < 40)] <- .5
  data_relief$lsm[which(data_relief$lsm<.5 & data_relief$x > 115 & data_relief$x < 135 & data_relief$y > 44 & data_relief$y < 50)] <- .5
  data_relief$lsm[which(data_relief$lsm<.5 & data_relief$x > 10 & data_relief$x < 40 & data_relief$y > 10 & data_relief$y < 14)] <- .5
  data_relief$lsm[which(data_relief$lsm<.5 & data_relief$x > 115 & data_relief$x < 121 & data_relief$y > 30 & data_relief$y < 33.5)] <- .5
  data_relief$lsm[which(data_relief$lsm<.5 & data_relief$x > 68.5 & data_relief$x < 70 & data_relief$y > 22.5 & data_relief$y < 26)] <- .5
  data_relief$lsm[which(data_relief$lsm<.5 & data_relief$x > 100 & data_relief$x < 102 & data_relief$y > 74 & data_relief$y < 76)] <- .5
  data_relief$lsm[which(data_relief$lsm<.5 & data_relief$x > 26 & data_relief$x < 30 & data_relief$y > 61 & data_relief$y < 69.5)] <- .5
  data_relief$lsm[which(data_relief$lsm<.5 & data_relief$x > 34 & data_relief$x < 40 & data_relief$y > 60 & data_relief$y < 63)] <- .5
  
  save(data_relief, file = "data/data_relief")
  
  
  
#############################################
########## Data for method section ##########
#############################################
  
  
  ## vpq850: southerly moisture transport at 850hPa
  ###################################################
  
  v850 <- load_data("v", 1979:2018, seq(28,36,.5), seq(67,85,.5), 850)
  q850 <- load_data("q", 1979:2018, seq(28,36,.5), seq(67,85,.5), 850)
  z850 <- load_data("z", 1979:2018, seq(28,36,1), seq(67,85,1), 850)
  
  # Selection of the points above the surface
  # sel_850 is refined compared to baudouin et al 2020a, by using Z850,
  # so to be sure that all points are above the surface for all timesteps
  # Baudouin et al 2020a used the constant threshold 15000dm

  # An interpolation is needed as I only downloaded Z at 1°
  m <- length(unique(z850$map$x))
  n <- length(unique(z850$map$y))
  test <- bilinear(1:m, 1:n, matrix(apply(z850$data, 1, min), m, n), rep(seq(1,m,.5), 2*n-1), rep(seq(1,n,.5), each=2*m-1))  
  sel_850 <- which(c(ag_era5(seq(28,36,.5), seq(67,85,.5))) < test$z) 
  
  # data list
  vpq850 <- v850[-1] # map and time data
  vpq850$sel <- sel_850
  vpq850$vpq <- v850$data*q850$data
  vpq850$vpq[vpq850$vpq < 0] <- 0
  vpq850$EOF <- prcomp(t(vpq850$vpq[vpq850$sel,]), scale=T, center=T); gc()
  
  save(vpq850, file="data/vpq850.Rdata")
  
  
  
  ## uvq700: moisture transport all direction at 700hPa
  #######################################################
  
  v700 <- load_data("v", 1979:2018, seq(28,36,.5), seq(67,85,.5), 700)
  u700 <- load_data("u", 1979:2018, seq(28,36,.5), seq(67,85,.5), 700)
  q700 <- load_data("q", 1979:2018, seq(28,36,.5), seq(67,85,.5), 700)
  
  z700 <- load_data("z", 1979:2018, seq(28,36,1), seq(67,85,1), 700)
  
  # Selection of the points above the surface
  m <- length(unique(z700$map$x))
  n <- length(unique(z700$map$y))
  test <- bilinear(1:m, 1:n, matrix(apply(z700$data, 1, min), m, n), rep(seq(1,m,.5), 2*n-1), rep(seq(1,n,.5), each=2*m-1))  
  sel_700 <- which(c(ag_era5(seq(28,36,.5), seq(67,85,.5))) < test$z)  

  # data list
  uvq700 <- v700[-1]
  uvq700$sel <- sel_700
  uvq700$u  <- u700$data
  uvq700$v  <- v700$data
  uvq700$q  <- q700$data 
  uvq700$uq <- u700$data*q700$data
  uvq700$vq <- v700$data*q700$data
  uvq700$EOF <- prcomp(t(rbind(uvq700$uq[uvq700$sel,], uvq700$vq[uvq700$sel,])), scale=F, center=T); gc()
  save(uvq700, file="data/uvq700.Rdata")
  

  
  ## vpq700: southerly moisture transport at 700hPa
  ###################################################
  
  ## Using data from uvq700
  vpq700 <- uvq700[c("time", "map", "sel")]
  vpq700$vpq <- uvq700$v*uvq700$q
  vpq700$vpq[vpq700$vpq < 0] <- 0
  vpq700$EOF <- prcomp(t(vpq700$vpq[vpq700$sel,]), scale=T, center=T); gc()  
  vpq700$EOF2 <- prcomp(t(vpq700$vpq[vpq700$sel,]), scale=F, center=T); gc()
  save(vpq700, file="data/vpq700.Rdata")
  



###########################################################
########## Data for section on wind contribution ##########
###########################################################

  ## predictors_WD (z300, dz300_500, dz500_700)
  ###############################################
  z300 <- load_data("z", 1979:2018, seq(20,50,1), seq(50,90,1), 300)
  z500 <- load_data("z", 1979:2018, seq(20,50,1), seq(50,90,1), 500)
  z700 <- load_data("z", 1979:2018, seq(20,50,1), seq(50,90,1), 700)
  
  dz300_500 <- z700
  dz300_500$data <- z300$data - z500$data
  
  dz500_700 <- z700
  dz500_700$data <- z500$data - z700$data
  
  z300$unseas      <- t(apply(z300$data,      1, unseas, ny = 40, h=4)); gc()
  dz300_500$unseas <- t(apply(dz300_500$data, 1, unseas, ny = 40, h=4)); gc()
  
  z300$sel <- which(c(ag_era5(seq(20,50,1), seq(50,90,1))) < apply(z300$data, 1, min))
  dz300_500$sel <- which(c(ag_era5(seq(20,50,1), seq(50,90,1))) < apply(z500$data, 1, min))
  dz500_700$sel <- which(c(ag_era5(seq(20,50,1), seq(50,90,1))) < apply(z700$data, 1, min))
  
  z300$EOF      <- prcomp(t(     z300$data), scale=T, center=T); gc()
  dz300_500$EOF <- prcomp(t(dz300_500$data), scale=T, center=T); gc()
  dz500_700$EOF <- prcomp(t(dz500_700$data[dz500_700$sel,]), scale=T, center=T); gc()
  
  save(z300, dz300_500, dz500_700, file="data/predictors_WD.Rdata")
  
  
  
  ## wind250
  ############
  
  u250 <- load_data2D("u", 1979:2018, seq(10,70,2), seq(-20,150,2), 250)
  v250 <- load_data2D("v", 1979:2018, seq(10,70,2), seq(-20,150,2), 250)
  wind250 <- list(data = sqrt(u250$data**2 + v250$data**2), 
                  map = u250$map, time = u250$time)
  wind250$unseas <- t(apply(wind250$data, 1, unseas, ny = 40, h=4)); gc()  
  save(wind250, file="data/wind250.Rdata")
  

  
  ## Vertical cross-section along 32°N
  ######################################
  z32 <- load_vert("z", 1979:2018, 32, c(55,80))
  z32$unseas <- apply(z32$data, c(1,3), unseas, ny = 40, h=4); gc()
  save(z32, file="data/vert_z32.Rdata"); rm(z32); gc()  
  
  t32 <- load_vert("t", 1979:2018, 32, c(55,80))
  t32$unseas <- apply(t32$data, c(1,3), unseas, ny = 40, h=4); gc()
  save(t32, file="data/vert_t32.Rdata"); rm(t32); gc()  
  
  v32 <- load_vert("v", 1979:2018, 32, c(55,80))
  v32$unseas <- apply(v32$data, c(1,3), unseas, ny = 40, h=4); gc()
  save(v32, file="data/vert_v32.Rdata"); gc()  
  
  q32 <- load_vert("q", 1979:2018, 32, c(55,80))
  q32$unseas <- apply(q32$data, c(1,3), unseas, ny = 40, h=4); gc()
  save(q32, file="data/vert_q32.Rdata"); gc()  
  
###############################################################
########## Data for section on moisture contribution ##########
###############################################################
  
  
  ## Precipitatble Water
  PW <- load_data2D("tcwv", 1979:2018, seq(10,50,1), seq(30,100,1), "surface")
  PW$data <- PW$data[,seq(1,350640,3)]
  PW$unseas <- t(apply(PW$data, 1, unseas, ny = 40, h=4)); gc()

  ## Vertically integrated moisture transport
  um <- load_data2D("p71.162", 1979:2018, seq(10,50,2), seq(30,100,2), "surface")
  um$data <- um$data[,seq(1,350640,3)]
  um$unseas <- t(apply(um$data, 1, unseas, ny = 40, h=4)); gc()

  vm <- load_data2D("p72.162", 1979:2018, seq(10,50,2), seq(30,100,2), "surface")
  vm$data <- vm$data[,seq(1,350640,3)]
  vm$unseas <- t(apply(vm$data, 1, unseas, ny = 40, h=4)); gc()
  save(PW, um, vm, file="data/data_PW.Rdata")

  ## Evaporation
  ET <- load_data2D("e", 1979:2018, seq(10,50,1), seq(30,100,1), "surface")
  ET$data[which(is.na(ET$data))] <- 0
  ET$data <- ET$data[,seq(1,350640,3)] + ET$data[,seq(1,350640,3)+1] + ET$data[,seq(1,350640,3)+2]
  ET$unseas <- t(apply(ET$data, 1, unseas, ny = 40, h=4)); gc()

  ## 10m wind
  u10 <- load_data2D("u10", 1979:2018, seq(10,50,2), seq(30,100,2), "surface")
  u10$data <- u10$data[,seq(1,350640,3)]
  u10$unseas <- t(apply(u10$data, 1, unseas, ny = 40, h=4)); gc()

  v10 <- load_data2D("v10", 1979:2018, seq(10,50,2), seq(30,100,2), "surface")
  v10$data <- v10$data[,seq(1,350640,3)]
  v10$unseas <- t(apply(v10$data, 1, unseas, ny = 40, h=4)); gc()
  save(ET, u10, v10, file="data/data_ET.Rdata")


  
  
  
