#####################################################################
#####################################################################
###################### Archived code for paper ######################
#####################################################################
#####################################################################

# Paper:   Synoptic processes of winter precipitation
#                 in the Upper Indus Basin
# Journal: Weather and Climate Dynamics (submitted)
# DOI:     #
# Authors: Jean-Philippe Baudouin, Michael Herzog, Cameron A. Petrie

####################################################################
# File purpose: Figures for section Results on the wind contribution 
# Note:        
# Version:     08/07/2021
#####################################################################
 
## libraries, sources
#######################

  source("01_functions.R")


## Functions
##############

  ## Random selection of "b" samples from a timeseries of "a" elements
  ## The size of the samples is "floor(a/b)" and each samples are disjointed
  Sample_split <- function(a,b=10) {
    set.seed(1991)
    n <- as.integer(a/b)
    samp <- c()
    for (i in 1:b){
      if (length(samp) == 0) {
        l <- 1:a
      } else {
        l <- (1:a)[-c(samp)]
      }
      samp <- cbind(samp, sample(l, n))
    }
    return(samp)
  }
  
  ## Function from Baudouin et al 2020a, to define the number of PC to select
  Pattern_extent <- function(data_map) {
    #data_map : data.frame(value, x, y)
  
    lat <- unique(data_map$x)
    step_lat <- min(abs(unique(lat[-1]-lat[-length(lat)])))
    if (length(step_lat) > 1) {
      stop("not regular latitudinal steps")
    }
    lon <- unique(data_map$y)
    step_lon <- min(abs(unique(lon[-1]-lon[-length(lon)])))
    if (length(step_lon) > 1) {
      stop("not regular longitudinal steps")
    }
  
    diff_list_next <- c()
    for (i in 1:length(data_map$y)) { # On same longitude
      pt_next <- which(data_map$y == data_map$y[i] & data_map$x == data_map$x[i] + step_lat)
      if (length(pt_next) == 1) {
        diff_list_next <- c(diff_list_next, (data_map$value[i] - data_map$value[pt_next])/step_lat)
      }
    }
    for (i in 1:length(data_map$y)) { # On same latitude
      pt_next <- which(data_map$y == data_map$y[i]+step_lon & data_map$x == data_map$x[i])
      if (length(pt_next) == 1) {
        diff_list_next <- c(diff_list_next, (data_map$value[i] - data_map$value[pt_next])/step_lon)
      }
    }
    
    print(sqrt(mean(data_map$value**2))/sqrt(mean(diff_list_next**2)))
    return(sqrt(mean(data_map$value**2))/sqrt(mean(diff_list_next**2)))
  }


  ## Linear fix of the prediction so it only allows positive values
  positive_fit <- function(X, Y, bias = F) {
    # X is the predictant (vector)
    # Y is the predictors (matrix)
    # if bias = T, mean(X) = mean(prediction)
    
    # The objective is to minimise the sum of square estimate of errors
    #   which is returned by the function SSE_reg, given the coefficients A
    # The function optim is then use to find the coeffients A the closest to the 
    #   normal linear regression, but minimising the SSE when only positive values 
    #   are considered. 
    # Using the coefficients of the linear regression as initial value ensure to
    #   get a better estimate than this one
    
    SSE_reg <- function(A, X, Y) {
      if (length(A) == 2) pred <- A[1] + Y*A[-1]
      else                pred <- A[1] + Y%*%A[-1]
      pred[which(pred<0)] <- 0
      if (! bias) pred <- pred/mean(pred)*mean(X)
      return(sum((X-pred)**2))
    }
  
    reg <- lm(X ~ Y)
    coef <- optim(coef(reg), SSE_reg, X=X, Y=Y)
    return(coef$par)
  }
  
  
## Data
#########

  load("data/Data_ERA5_EOF_all3.Rdata") # See code for baudouin et al 2020a at https://zenodo.org/record/3765211
  
# From Data.R
  load("data/vpq700.Rdata")
  load("data/vpq850.Rdata")
  load("data/uvq700.Rdata")
  load("data/precip_ERA5_3h.Rdata") 

  
  # Winter Month: October to May
  sel_win <- which(as.numeric(format(pr_ind_era5_3h$time,
                                     format="%m")) %in% c(10:12, 1:5))
  sel_win <- sel_win[-c(1,2,length(sel_win))]


  
################################################################################
################################################################################  
  
  
####################################################
########### R² for different regressions ###########
####################################################


## Same as in baudouin et al. 2020a:  
#####################################
  
  sel_win2 <- which(as.numeric(format(
    pr_ind_era5_3h$time[sel_noNA_EOF], format="%m")) %in% c(10:12, 1:5))
  n <- dim(X_EOF)[1]
  Y <- X_EOF[sel_win2,1:sum(l[1:2])]
  X <- pr_ind_era5_3h$up[sel_noNA_EOF[sel_win2]] 
  l <- l[1:2]
  ss <- Sample_split(length(sel_win2),10)  
  R2_cv <- c()
  for (i in 1:10) { # R² is computed with 10 different sampling 
    reg <- lm(X[-ss[,i]] ~ Y[-ss[,i],])
    R2_cv <- c(R2_cv, 1-var(Y[ss[,i],]%*%coef(reg)[-1] - X[ss[,i]])/
      var(X[ss[,i]]))
  }
  mean(R2_cv) # 0.8922661


## Smaller area
#################
  # EP (extent of pattern) needs to be recomputed
  list_extent <- c()
  for (k in 1:100) { # 850hPa southerly wind
    list_extent <- c(list_extent,Pattern_extent(data.frame(
      value = vpq850$EOF$rotation[,k],
      x = vpq850$map[vpq850$sel,1],
      y = vpq850$map[vpq850$sel,2])))
  }
  smooth850 <- ksmooth(1:100,list_extent, band=10)$y
  k_850 <- max(which(smooth850>0.73)) #22

  list_extent <- c()
  for (k in 1:100) { # 700hPa southerly wind
    list_extent <- c(list_extent,Pattern_extent(data.frame(
      value = vpq700$EOF$rotation[,k],
      x = vpq700$map[vpq700$sel,1],
      y = vpq700$map[vpq700$sel,2])))
  }
  smooth700 <- ksmooth(1:100,list_extent, band=10)$y
  k_700 <- max(which(smooth700>0.73)) # 27

  # Regression
  X <- pr_ind_era5_3h$up[sel_win]
  ss <- Sample_split(length(sel_win),10) 
  R2_cv <- c()
  for (i in 1:10) {
    Y850 <- vpq850$EOF$x[sel_win,1:k_850]
    Y700 <- vpq700$EOF$x[sel_win,1:k_700]
    reg <- lm(X[-ss[,i]] ~ Y850[-ss[,i],] + Y700[-ss[,i],])
    R2_cv <- c(R2_cv, 1-var(cbind(Y850,Y700)[ss[,i],]%*%
      coef(reg)[-1] - X[ss[,i]])/var(X[ss[,i]]))
  }
  mean(R2_cv) # 0.8886685


## Only vpq700 (Southerly moisture transport at 700hPa)
#########################################################
  Y700 <- vpq700$EOF$x[sel_win,1:k_700]
  R2_cv <- c()
  for (i in 1:10) {
    reg <- lm(X[-ss[,i]] ~ Y700[-ss[,i],])
    R2_cv <- c(R2_cv, 1-var(Y700[ss[,i],]%*%coef(reg)[-1] -
      X[ss[,i]])/var(X[ss[,i]]))
  }
  mean(R2_cv) # 0.8543192

  
## vpq700 just centered in PCA
################################
  list_extent <- c()
  for (k in 1:100) {
    list_extent <- c(list_extent,Pattern_extent(data.frame(
      value = vpq700$EOF2$rotation[,k],
      x = vpq700$map[vpq700$sel,1],
      y = vpq700$map[vpq700$sel,2])))
  }
  smooth700_2 <- ksmooth(1:100,list_extent, band=10)$y
  k_700_2 <- max(which(smooth700_2>0.73)) # 22
  

  Y700 <- vpq700$EOF2$x[sel_win,1:k_700_2]
  R2_cv <- c()
  for (i in 1:10) {
    reg <- lm(X[-ss[,i]] ~ Y700[-ss[,i],])
    R2_cv <- c(R2_cv, 1-var(Y700[ss[,i],]%*%coef(reg)[-1] -
                              X[ss[,i]])/var(X[ss[,i]]))
  }
  mean(R2_cv) # 0.8461803
  
  
## Same predictors, but fixing negative prediction to 0 when computing R²
###########################################################################
  R2_cv <- c()
  for (i in 1:10) {
    reg <- lm(X[-ss[,i]] ~ Y700[-ss[,i],])
    R2_cv <- c(R2_cv, 1-var(pmax(Y700[ss[,i],]%*%coef(reg)[-1] + coef(reg)[1],0) -
                              X[ss[,i]])/var(X[ss[,i]]))
  }
  mean(R2_cv) # 0.8510808
  
  
  
## Same predictors, but tuning the prediction for positive values
###################################################################
  R2_cv <- c()
  for (i in 1:10) {
    coef <- positive_fit(X, Y700)
    R2_cv <- c(R2_cv, 1-var(pmax(Y700[ss[,i],]%*%coef[-1] + coef[1],0) -
      X[ss[,i]])/var(X[ss[,i]]))
  }
  mean(R2_cv) # 0.8534794



## New predictor: Moisture transport from all direction at 700hPa
###################################################################
  # A new EP is needed for the predictor
  list_extent <- c()
  m <- length(uvq700$map[uvq700$sel,1])
  for (k in 1:100) {
    list_extent <- c(list_extent,Pattern_extent(data.frame(
        value = uvq700$EOF$rotation[1:m,k],
        x = uvq700$map[uvq700$sel,1],
        y = uvq700$map[uvq700$sel,2]))/2 +
      Pattern_extent(data.frame(
        value = uvq700$EOF$rotation[1:m+m,k],
        x = uvq700$map[uvq700$sel,1],
        y = uvq700$map[uvq700$sel,2]))/2)
  }
  smooth_uvq <- ksmooth(1:100,list_extent, band=10)$y
  k_uvq <- max(which(smooth_uvq>0.73)) # 46

  # R2
  Y700 <- uvq700$EOF$x[sel_win,1:k_uvq]
  R2_cv <- c()
  for (i in 1:10) {
    coef <- positive_fit(X, Y700)
    reg <- lm(X[-ss[,i]] ~ Y[-ss[,i],])
    R2_cv <- c(R2_cv, 1-var(pmax(Y700[ss[,i],]%*%coef[-1] + coef[1],0) -
                              X[ss[,i]])/var(X[ss[,i]]))
  }
  mean(R2_cv) # 0.8780457

  
  
  
## Computing the R² from the product of wind and moisture contribution
########################################################################
  
  # We compute R² with various method (see below)
  R2_cv1 <- c()
  R2_cv2 <- c()
  R2_cv3 <- c()
  R2_cv4 <- c()
  for (i in 1:10) {
    print(i)
    coef <- positive_fit(X, Y700)
    
    # wind and moisture contributions
    uv <- t(uvq700$u[uvq700$sel, sel_win])%*%
      (uvq700$EOF$rotation[1:m,1:k_uvq]%*%coef[-1]) +
          t(uvq700$v[uvq700$sel, sel_win])%*%
      (uvq700$EOF$rotation[1:m+m,1:k_uvq]%*%coef[-1])
    q <- t(uvq700$q[uvq700$sel, sel_win])%*%
      sqrt((uvq700$EOF$rotation[1:m,1:k_uvq]%*%coef[-1])**2 +
           (uvq700$EOF$rotation[1:m+m,1:k_uvq]%*%coef[-1])**2)
    
    uvq <- uv*q
    reg2 <- lm(X[-ss[,i]] ~ uvq[-ss[,i],] + I(uvq[-ss[,i],]**2))
    uvq_fit <- coef(reg2)[1] + coef(reg2)[2]*uvq + 
      coef(reg2)[3]*uvq**2
    R2_cv1 <- c(R2_cv1, cor(X[ss[,i]], uvq[ss[,i]])**2)
    R2_cv2 <- c(R2_cv2, cor(X[ss[,i]], pmax(uvq[ss[,i]],0))**2)
    R2_cv3 <- c(R2_cv3, cor(X[ss[,i]], uvq_fit[ss[,i]])**2)
    R2_cv4 <- c(R2_cv4, cor(X[ss[,i]], pmax(uvq_fit[ss[,i]],0))**2)
  }
  
  mean(R2_cv1) # 0.7985733 # Normal
  mean(R2_cv2) # 0.8437976 # fixing negative values of wind to 0
  mean(R2_cv3) # 0.8472089 # Quadratic fit
  mean(R2_cv4) # 0.8532704 # Quadratic fit / non-negative wind values


  
  
################################################################################
################################################################################

  
##################################################
### Figure 1: Pattern (A) and scatter plot (B) ###
##################################################

  ## Regression
  n <- 46 # k_uvq, number of PC selected
  m <- length(uvq700$sel)
  X <- pr_ind_era5_3h$up[sel_win]
  Y <- uvq700$EOF$x[sel_win,1:n]
  coef <- positive_fit(X, Y)

  
  
  ## Pattern of the regression coefficient
  data_reg <- data.frame(u = uvq700$EOF$rotation[1:m,1:n]%*%coef[1:n+1],
                         v = uvq700$EOF$rotation[1:m+m,1:n]%*%coef[1:n+1],
                         x = uvq700$map[uvq700$sel,1],
                         y = uvq700$map[uvq700$sel,2])
  data_reg$value <- sqrt(data_reg$u**2 + data_reg$v**2) 
  
  facW1 <- 1

  p_map <- ggplot() +
    geom_raster(data=data_reg,   aes(x, y, fill=value), interpolate = F) +
    theme_map + 
    coord_quickmap() + 
    labs(title = NULL, y = "latitude (in degree N)", x = "longitude (in degree W)") +
    scale_x_continuous(expand=c(0,-1), limits=c(65.75,86.25), breaks=seq(67,85,1)) + 
    scale_y_continuous(expand=c(0,-1), limits=c(26.75,37.25), breaks=seq(28,36,1)) + 
    geom_segment(data=data_reg[,],aes(x = x, y = y, xend=x+u*facW1, yend=y+v*facW1),
                 arrow = arrow(length = unit(0.15,"cm")), color="white", size=1) +
    mask_up(size=1) +
    labs(x=NULL, y=NULL, title=NULL) +
    scale_fill_gradientn(colours = c("white", "#feb24c", "#bd0026"),
                         na.value = "white",
                         guide = guide_colorbar(barwidth = 15, barheight = 1, ticks = F), 
                         name = NULL,
                         limits = c(0,1)*max(abs(data_reg$value)),
                         breaks = c(0,0.5)*max(abs(data_reg$value)),
                         labels = c("0",as.expression(bquote("||"~beta~"||"[~2])))) +
    annotate("label", x=83.75, y=35.2, label="B)", size=10, label.padding = unit(c(.6), "lines"))                   

  
  
  ## timeseries for the scatterplots
  ts_pred <- coef[1] + c(coef[-1]%*%t(uvq700$EOF$x[,1:n]))
  ts_pred0 <- ts_pred
  ts_pred0[ts_pred0 < 0] <- 0
  
  coef2 <- sqrt((uvq700$EOF$rotation[1:m,1:n]%*%coef[-1])**2 +
                  (uvq700$EOF$rotation[1:m+m,1:n]%*%coef[-1])**2)
  
  ts_uv <- (t(uvq700$u[uvq700$sel, ])%*%
              (uvq700$EOF$rotation[1:m,1:n]%*%coef[-1]) +
              t(uvq700$v[uvq700$sel, ])%*%
              (uvq700$EOF$rotation[1:m+m,1:n]%*%coef[-1]))/sum(coef2) # W700
  ts_uv0 <- ts_uv
  ts_uv0[ts_uv0 < 0] <- 0
  ts_q <- (t(uvq700$q[uvq700$sel, ])%*%coef2)/sum(coef2) # Q700
  
  # The timeseries are saved for other analyses
  save(ts_uv, ts_uv0, ts_q, ts_pred, file="data/ts_uvq.Rdat")
  
  
  
  ## Scatterplot
  data_pt <- data.frame(v=pr_ind_era5_3h$up, z=ts_uv0*ts_q)
  p_pts <- ggplot(data_pt[sel_win,]) +
           geom_point(aes(x=v, y=z), size=.5) +
           geom_hline(aes(yintercept=0), color="grey20", size=1) +
           geom_vline(aes(xintercept=0), color="grey20", size=1) +
           theme_plot +
           scale_y_continuous(expand = c(0,0), limits=c(0,0.06), breaks=seq(0,0.06,0.02)) +
           scale_x_continuous(expand = c(0,0), limits=c(0,6), breaks=seq(0,6,2)) +
           labs(x="Upper Indus precipitation (mm per 3hours)", y=as.expression(bquote(atop("Product wind-moisture", "contributions (m."*s^"-1"*".kg.kg"^"-1"*")")))) +
           #annotate("label", x=4.5, y=.55, label=paste("R^2 == ", R2), parse=T, size=7, label.padding = unit(c(.4), "lines"), label.size = 0) +
           annotate("label", x=.9, y=0.0525, label="C)", size=10, label.padding = unit(c(.6), "lines"))

  # computing linear fit
  reg <- lm(v ~ z, data=data_pt[sel_win,])
  y <- seq(0,0.06,0.0001)
  x <- coef(reg)[1] + coef(reg)[2]*y

  # computing quadratic fit
  reg <- lm(v ~ z + I(z**2), data=data_pt[sel_win,])
  x2 <- coef(reg)[1] + coef(reg)[2]*y + coef(reg)[3]*y**2
  data_fit <- data.frame(x=c(x, x2), y=rep(y, 2), type=rep(1:2, each=length(y)))

  p_pts <- p_pts + geom_line(data=data_fit, aes(x=x, y=y, group=type, colour=as.factor(type)), size=1) +
  scale_colour_manual(values=c("darkgreen", "darkolivegreen3"), labels=c("linear fit", "quadratic fit"), name=NULL)
  
  p <- egg::ggarrange(p_map, p_pts, nrow=1, widths=c(5, 3), draw=F)
  ggsave("figures/Fig01_Reg_pr_uvq700.pdf", p, device="pdf", width=12, height=5) 
  
  
  
  ## Introductory map with elevation
  ####################################
  require(elevatr)
  require(cptcity)

  
  ## General variables
  # map projection
  ll_prj <- "+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs" 
  # lat lon grid used for plot
  lon <- seq(60, 90, .1)
  lat <- seq(15, 45, .1)
  
  # loading altitude data
  elev  <- get_elev_point(data.frame(x=rep(lon, length(lat)), y=rep(lat, each = length(lat))), ll_prj, src = "aws", z = 5)
  elev2   <- as.data.frame(elev)
  
  x <- seq(0,1,0.01) # used to scale the colour bar
  
  p2 <- ggplot(elev2, aes(x=x, y=y)) + 
    geom_raster(aes(fill = pmax(pmin(elevation,6000), 0)), interpolate=T) +
    geom_raster(data = elev2[which(elev2$elevation<0 & elev2$y < 30),],
                 aes(x = x, y = y), fill = "steelblue4") +
    geom_contour(data = elev2[which(elev2$y < 30),], aes(z=elevation), breaks=0, colour = "black", size=1) +
    geom_contour(data=mask, aes(x=x, y=y, z=(up)), breaks=.5, colour = "black", size=2, linetype = "31") +
    theme_map + coord_quickmap() +
    theme(legend.position = "bottom") +
    labs(x=NULL, y=NULL, title=NULL) +   
    scale_x_continuous(expand=c(0,0), limits=c(60, 90), breaks = seq(-50,100,5)) +
    scale_y_continuous(expand=c(0,0), limits=c(15, 45)) +
    scale_fill_gradientn(
      colours=cpt(pal = "td_DEM_poster", n = 101,
                  colorRampPalette = FALSE, rev = FALSE),
      na.value = "white", limits = c(0, 6000),
      guide = guide_colourbar(title.position = "top", title.hjust=.5,
                              barwidth = 20, barheight = 1, 
                              ticks.colour = "black", ticks.linewidth = 2),
      name = "Altitude (m)",
      breaks = c(0, 1000, 2000, 4000, 6000),
      values = (x**2/2 - x**10/10)/.4)
  
  ggsave("figures/Fig00_relief.pdf", p2, device="pdf", width=8.3, height=8) 

      
